@@ -10,204 +10,10 @@ import requests
 import sys
 import time
 
-from lhub import exceptions
-from lhub.log import Logger
-
-
-class URLs:
-    _version = 0
-
-    def __init__(self, server_name):
-        self.server_name = str(server_name).lower().strip()
-
-    @property
-    def base(self):
-        return f"https://{self.server_name}"
-
-    @property
-    def alert_fetch(self):
-        return f"{self.base}/api/alert/alert-{{}}"
-
-    @property
-    def alerts_search_advanced(self):
-        return f"{self.base}/api/alert/search/advanced"
-
-    @property
-    def alerts_search_basic(self):
-        return f"{self.base}/api/alert/search/basic"
-
-    @property
-    def alerts_search_validate(self):
-        return f"{self.base}/api/alert/search/validate"
-
-    @property
-    def baselines(self):
-        return f"{self.base}/api/content-management/content/baseline"
-
-    @property
-    def batch_reprocess(self):
-        return f"{self.base}/api/demo/batch-{{}}"
-
-    @property
-    def batch_results_by_id(self):
-        return f"{self.base}/api/demo/batch-{{}}/correlations"
-
-    @property
-    def _case_base_url(self):
-        return f"{self.base}/api/case/{{case_id}}"
-
-    @property
-    def case_get_basic_details(self):
-        return f"{self._case_base_url}/basicDetails"
-
-    @property
-    def case_get_custom_fields(self):
-        return f"{self._case_base_url}/fields"
-
-    @property
-    def case_update_linked_alerts(self):
-        return self._case_base_url
-
-    @property
-    def case_status_list_workflows(self):
-        return f"{self.base}/api/cases/status-workflow"
-
-    @property
-    def case_status_workflow_by_id(self):
-        return f"{self.base}/api/cases/status-workflow/statusworkflow-{{}}"
-
-    @property
-    def cases_get_prefix(self):
-        return f"{self.base}/api/cases/prefix"
-
-    @property
-    def command(self):
-        return f"{self.base}/commands/{{}}"
-
-    @property
-    def command_execute(self):
-        return f"{self.base}/api/commands/execute"
-
-    @property
-    def connection_status(self):
-        return f"{self.base}/api/connection/status"
-
-    @property
-    def connections(self):
-        return f"{self.base}/api/content-management/content/connection"
-
-    @property
-    def custom_list_data(self):
-        return f"{self.base}/api/lists/{{}}/data"
-
-    @property
-    def custom_list_data_with_filtering(self):
-        return f"{self.base}/api/lists/{{}}/search"
-
-    @property
-    def custom_lists(self):
-        return f"{self.base}/api/content-management/content/customList"
-
-    @property
-    def event_types(self):
-        # Not sure when the old one changed, but it doesn't work in 70
-        if self._version < 70:
-            return f"{self.base}/api/eventtype-flowrefs"
-        return f"{self.base}/api/content-management/content/eventType"
-
-    @property
-    def fields(self):
-        return f"{self.base}/api/fields"
-
-    @property
-    def fields_get_computed(self):
-        # Fetch the config of computed fields, i.e. lh_url, lh_filehash, and lh_ipaddress (example: regex exclusions)
-        return f"{self.base}/api/fields/computed"
-
-    @property
-    def flow_export(self):
-        return f"{self.base}/api/flow/flow-{{}}/export"
-
-    @property
-    def integrations(self):
-        return f"{self.base}/api/content-exchange/browse/Integration"
-
-    @property
-    def login(self):
-        return f"{self.base}/api/login"
-
-    @property
-    def logout(self):
-        return f"{self.base}/api/logout"
-
-    @property
-    def me(self):
-        return f"{self.base}/api/user-management/me"
-
-    @property
-    def modules(self):
-        return f"{self.base}/api/content-exchange/browse/Module"
-
-    @property
-    def notebooks_attach(self):
-        return f"{self.base}/api/notebook/attachNotebooksToEntity"
-
-    @property
-    def notebooks_attached(self):
-        return f"{self.base}/api/notebook/attachedNotebooks"
-
-    @property
-    def notebooks_list(self):
-        return f"{self.base}/api/content-management/content/notebook"
-
-    @property
-    def playbooks_list(self):
-        return f"{self.base}/api/content-management/content/playbook"
-
-    @property
-    def rule_set(self):
-        return f"{self.base}/api/demo/ruleSet-{{}}"
-
-    @property
-    def rule_sets(self):
-        return f"{self.base}/api/demo/hub/ruleSets"
-
-    @property
-    def stream_batches(self):
-        return f"{self.base}/api/stream/stream-{{}}/batches/filter"
-
-    @property
-    def stream_by_id(self):
-        return f"{self.base}/api/stream/stream-{{}}"
-
-    @property
-    def stream_states(self):
-        return f"{self.base}/api/stream/states"
-
-    @property
-    def stream_pause(self):
-        return f"{self.base}/api/stream/pause"
-
-    @property
-    def stream_resume(self):
-        return f"{self.base}/api/stream/resume"
-
-    @property
-    def streams(self):
-        return f"{self.base}/api/content-management/content/stream"
-
-    @property
-    def user_groups(self):
-        return f"{self.base}/api/user-management/user-group/search"
-
-    @property
-    def users(self):
-        return f"{self.base}/api/user-management/user/search"
-
-    @property
-    def version(self):
-        return f"{self.base}/api/version"
-
+from . import exceptions
+from .log import Logger
+from .url import URLs
+from .common import helpers
 
 cached_obj = namedtuple('CachedObject', ['time', 'value'])
 
@@ -350,7 +156,7 @@ class LogicHubAPI:
     @property
     def notebooks(self):
         if not self.__get_cached_object(self.__notebooks):
-            result = self.notebooks_list()
+            result = self.list_notebooks()
             self.notebooks = result["result"]["data"]["data"]
         return self.__notebooks.value
 
@@ -518,81 +324,35 @@ class LogicHubAPI:
             results = results["data"]
         return results
 
-    def get_event_types(self, limit=25):
-        limit = int(limit or 25)
-        if self.version < 70:
-            return self.__get_event_types_v1(limit=limit)
-        return self.__get_event_types_v2(limit=limit)
+    # ToDo Revisit and finish this
+    # def get_alerts(self, advanced_filter=None, limit=100):
+    #     limit = int(limit) if limit is not None else 100_000
+    #     body = {
+    #         "pageNumber": 0,
+    #         "pageSize": limit,
+    #         "query": advanced_filter,
+    #         "sortCol": "id",
+    #         "sortOrder": "desc"
+    #     }
+    #     connection_list = []
+    #     if not isinstance(connections, (list, tuple)):
+    #         connections = [connections]
+    #     for connection in connections:
+    #         if isinstance(connection, str) and connection.startswith("connection-"):
+    #             connection_list.append(connection)
+    #         else:
+    #             connection_list.append(f"connection-{connection}")
+    #
+    #     headers = {"Content-Type": "application/json"}
+    #     # response = self._http_request(method="POST", url=self.url.connection_status, headers=headers, body=json.dumps(connection_list))
+    #     response = self._http_request(method="POST", url=self.url.connection_status, headers=headers, body=connection_list)
+    #     return response.json()
 
-    # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
-    def get_rule_sets(self, limit=25):
-        limit = int(limit or 25)
-        params = {
-            "fields": "name,isPublic",
-            "pageSize": limit,
-        }
-        response = self._http_request(url=self.url.rule_sets, params=params)
-        results = response.json()
-        try:
-            results = results["result"]["data"]
-        except Exception:
-            raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing rule sets")
-        return results
-
-    @staticmethod
-    def __sanitize_input_rule_set(rule_set_id):
-        if isinstance(rule_set_id, int):
-            return rule_set_id
-        rule_set_num_str = re.sub(r'\D+', '', str(rule_set_id))
-        if not rule_set_num_str:
-            raise ValueError("Invalid rule set ID")
-        return int(rule_set_num_str)
-
-    @staticmethod
-    def __sanitize_input_rule_field_mappings(field_mappings):
-        if not isinstance(field_mappings, dict):
-            try:
-                field_mappings = json.loads(field_mappings)
-            except Exception:
-                raise exceptions.Formatting.InvalidRuleFormat(field_mappings)
-        if not field_mappings:
-            raise exceptions.Formatting.InvalidRuleFormat(field_mappings)
-        return field_mappings
-
-    @staticmethod
-    def __sanitize_input_rule_score(score, round_points: int = None):
-        try:
-            score = float(score)
-            assert 10 >= score >= 0
-        except (ValueError, TypeError, AssertionError):
-            raise ValueError("Score must be a number between 0 and 10")
-        if round_points:
-            score = round(score, round_points)
-        return score
-
-    def get_stream_by_id(self, stream_id: int):
-        headers = {"Accept": "application/json"}
-        # ToDo Revisit this. I better approach might be:
-        # try:
-        #     response = self._http_request(url=self.url.stream_by_id.format(stream_id), headers=headers)
-        #     return response.json()
-        # except requests.exceptions.HTTPError:
-        #     if self.last_response_status == 400 and 'Cannot find entity for id StreamId' in self.last_response_text:
-        #         raise exceptions.StreamNotFound(stream_id)
-        response = self._http_request(url=self.url.stream_by_id.format(stream_id), headers=headers, test_response=False)
-        try:
-            response.raise_for_status()
-        except requests.exceptions.HTTPError:
-            if response.status_code == 400 and 'Cannot find entity for id StreamId' in response.text:
-                raise exceptions.StreamNotFound(stream_id)
-        return response.json()
-
-    def get_stream_states(self, stream_ids: list):
-        new_stream_id_list = []
-        for s in stream_ids:
-            new_stream_id_list.append(f'stream-{s}')
-        body = {"streams": new_stream_id_list}
-        response = self._http_request(method="POST", url=self.url.stream_states, body=body)
+    def get_batch_results_by_id(self, batch_id: int, limit=1000, offset=0):
+        limit = int(limit or 1000)
+        offset = int(offset or 0)
+        params = {"fields": "*", "pageSize": limit, "after": offset, "cachedOnly": "true"}
+        response = self._http_request(url=self.url.batch_results_by_id.format(int(batch_id)), params=params)
         return response.json()
 
     def get_batches_by_stream_id(self, stream_id, limit=25, offset=0, statuses=None, exclude_empty_results=False):
@@ -606,97 +366,66 @@ class LogicHubAPI:
         response = self._http_request(method="POST", url=self.url.stream_batches.format(stream_id), params=params, body=body)
         return response.json()
 
-    def get_batch_results_by_id(self, batch_id: int, limit=1000, offset=0):
-        limit = int(limit or 1000)
+    def get_connection_status(self, connections: list):
+        """
+        Get status of one or more connections
+        :param connections: list of connection IDs, either as ints (like [6, 8]) or in full form (like ["connection-6", "connection-8"])
+        :return:
+        """
+        connection_list = []
+        if not isinstance(connections, (list, tuple)):
+            connections = [connections]
+        for connection in connections:
+            if isinstance(connection, str) and connection.startswith("connection-"):
+                connection_list.append(connection)
+            else:
+                connection_list.append(f"connection-{connection}")
+
+        headers = {"Content-Type": "application/json"}
+        # response = self._http_request(method="POST", url=self.url.connection_status, headers=headers, body=json.dumps(connection_list))
+        response = self._http_request(method="POST", url=self.url.connection_status, headers=headers, body=connection_list)
+        return response.json()
+
+    # ToDo in progress
+    def get_custom_list_data(self, list_id, filter_sql: str = None, limit: int = None, offset: int = None):
+        """
+
+        :param list_id: numeric ID for requested custom list
+        :param filter_sql: Optional: filter custom list data to return (spark SQL)
+        :param limit: Optional: limit the number of results to return (default is 10,000)
+        :param offset: Optional: if using pagination, provide the entry number to fetch (default is 0)
+        :return:
+        """
+        assert isinstance(list_id, int) or int(list_id)
+        list_id = int(list_id)
+        if limit:
+            assert isinstance(limit, int) or int(limit)
+        if offset:
+            assert isinstance(offset, int) or int(offset)
+        limit = int(limit or 10_000)
         offset = int(offset or 0)
-        params = {"fields": "*", "pageSize": limit, "after": offset, "cachedOnly": "true"}
-        response = self._http_request(url=self.url.batch_results_by_id.format(int(batch_id)), params=params)
-        return response.json()
-
-    def get_integrations(self):
-        response = self._http_request(method="GET", url=self.url.integrations)
-        return response.json()
-
-    def get_streams(self, search_text: str = None, filters: list = None, limit: int = 25, offset: int = 0):
-        params = {"libraryView": "all"}
-        headers = {"Content-Type": "application/json"}
-
-        filters = filters or []
-        if search_text:
-            filters.append({"searchText": search_text})
-
-        body_dict = {
-            "filters": filters,
-            "offset": offset,
-            "pageSize": limit,
-            "sortColumn": "name",
-            "sortOrder": "ASC"
-        }
-
-        response = self._http_request(method="POST", url=self.url.streams, body=body_dict, headers=headers, params=params)
-        return response.json()
-
-    def get_version_info(self):
-        self.log.debug("Fetching LogicHub version")
-        try:
-            response = self._http_request(
-                url=self.url.version,
-                reauth=False,
-                timeout=self.http_timeout_default)
-            response_dict = response.json()
-        except (KeyError, ValueError, TypeError, IndexError):
-            raise exceptions.LhBaseException("LogicHub version could not be established")
+        if filter_sql:
+            url = self.url.custom_list_data_with_filtering.format(list_id)
+            method = "POST"
+            payload = filter_sql.strip()
         else:
-            # Update version information any time this api call is run successfully
-            self.version_info = response_dict
-            self.version = float(re.match("m(.*)", self.version_info["version"]).group(1))
-        return response_dict
-
-    # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
-    def get_rules_for_rule_set(self, rule_set):
-        rule_set = LogicHubAPI.__sanitize_input_rule_set(rule_set)
-        params = {
-            "fields": "name,isPublic,rules[filter,values*1000,score]*1000",
-        }
-        response = self._http_request(url=self.url.rule_set.format(rule_set), params=params)
-        results = response.json()
-        try:
-            results = results["result"]["rules"]["data"]
-        except Exception:
-            raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing rules of a rule set")
-        return results
-
-    # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
-    # ToDo Create an action method for this, and update the beta integration action to use it
-    def add_scoring_rule(self, rule_set, field_mappings, score: float or str):
-        rule_set = LogicHubAPI.__sanitize_input_rule_set(rule_set)
-        field_mappings = LogicHubAPI.__sanitize_input_rule_field_mappings(field_mappings)
-        score = LogicHubAPI.__sanitize_input_rule_score(score)
-        headers = {"Content-Type": "application/json"}
-        body = {
-            "method": "addRule",
-            "parameters": {
-                "values": [field_mappings],
-                "score": score
-            }
-        }
-        response = self._http_request(url=self.url.rule_set.format(rule_set), method="POST", body=body, headers=headers)
+            url = self.url.custom_list_data
+            method = "GET"
+            payload = None
+        response = self._http_request(
+            method=method,
+            url=url,
+            params={"offset": offset, "limit": limit},
+            body=payload,
+            headers={'Content-Type': 'application/json'},
+        )
         results = response.json()
         try:
             results = results["result"]
         except Exception:
-            raise exceptions.UnexpectedOutput("API response does not match the expected schema for adding a scoring rule")
+            raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing custom lists")
         return results
 
-    def get_rule_set_by_name(self, name):
-        rule_sets = self.get_rule_sets()
-        rule_set = [x for x in rule_sets if x['name'] == name]
-        if not rule_set:
-            raise exceptions.RuleSetNotFound(f"No rule set found matching name: {name}")
-        rule_set = rule_set[0]
-        rule_set['rules'] = self.get_rules_for_rule_set(rule_set['id'])
-        return rule_set
-
     def get_custom_lists(self, search_text: str = None, filters: list = None, limit: int = None, offset: int = None, verify_results=True):
         """
 
@@ -738,44 +467,124 @@ class LogicHubAPI:
                 raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing custom lists")
         return results
 
-    # ToDo in progress
-    def get_custom_list_data(self, list_id, filter_sql: str = None, limit: int = None, offset: int = None):
-        """
+    def get_event_types(self, limit=25):
+        limit = int(limit or 25)
+        if self.version < 70:
+            return self.__get_event_types_v1(limit=limit)
+        return self.__get_event_types_v2(limit=limit)
 
-        :param list_id: numeric ID for requested custom list
-        :param filter_sql: Optional: filter custom list data to return (spark SQL)
-        :param limit: Optional: limit the number of results to return (default is 10,000)
-        :param offset: Optional: if using pagination, provide the entry number to fetch (default is 0)
-        :return:
-        """
-        assert isinstance(list_id, int) or int(list_id)
-        list_id = int(list_id)
-        if limit:
-            assert isinstance(limit, int) or int(limit)
-        if offset:
-            assert isinstance(offset, int) or int(offset)
-        limit = int(limit or 10_000)
-        offset = int(offset or 0)
-        if filter_sql:
-            url = self.url.custom_list_data_with_filtering.format(list_id)
-            method = "POST"
-            payload = filter_sql.strip()
+    def get_integrations(self):
+        response = self._http_request(method="GET", url=self.url.integrations)
+        return response.json()
+
+    def get_rule_set_by_name(self, name):
+        rule_sets = self.get_rule_sets()
+        rule_set = [x for x in rule_sets if x['name'] == name]
+        if not rule_set:
+            raise exceptions.RuleSetNotFound(f"No rule set found matching name: {name}")
+        rule_set = rule_set[0]
+        rule_set['rules'] = self.get_rules_for_rule_set(rule_set['id'])
+        return rule_set
+
+    # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
+    def get_rule_sets(self, limit=25):
+        limit = int(limit or 25)
+        params = {
+            "fields": "name,isPublic",
+            "pageSize": limit,
+        }
+        response = self._http_request(url=self.url.rule_sets, params=params)
+        results = response.json()
+        try:
+            results = results["result"]["data"]
+        except Exception:
+            raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing rule sets")
+        return results
+
+    # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
+    def get_rules_for_rule_set(self, rule_set):
+        rule_set = helpers.sanitize_input_rule_set_id(rule_set)
+        params = {
+            "fields": "name,isPublic,rules[filter,values*1000,score]*1000",
+        }
+        response = self._http_request(url=self.url.rule_set.format(rule_set), params=params)
+        results = response.json()
+        try:
+            results = results["result"]["rules"]["data"]
+        except Exception:
+            raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing rules of a rule set")
+        return results
+
+    def get_stream_by_id(self, stream_id: int):
+        headers = {"Accept": "application/json"}
+        # ToDo Revisit this. I better approach might be:
+        # try:
+        #     response = self._http_request(url=self.url.stream_by_id.format(stream_id), headers=headers)
+        #     return response.json()
+        # except requests.exceptions.HTTPError:
+        #     if self.last_response_status == 400 and 'Cannot find entity for id StreamId' in self.last_response_text:
+        #         raise exceptions.StreamNotFound(stream_id)
+        response = self._http_request(url=self.url.stream_by_id.format(stream_id), headers=headers, test_response=False)
+        try:
+            response.raise_for_status()
+        except requests.exceptions.HTTPError:
+            if response.status_code == 400 and 'Cannot find entity for id StreamId' in response.text:
+                raise exceptions.StreamNotFound(stream_id)
+        return response.json()
+
+    def get_stream_states(self, stream_ids: list):
+        new_stream_id_list = []
+        for s in stream_ids:
+            new_stream_id_list.append(f'stream-{s}')
+        body = {"streams": new_stream_id_list}
+        response = self._http_request(method="POST", url=self.url.stream_states, body=body)
+        return response.json()
+
+    def get_version_info(self):
+        self.log.debug("Fetching LogicHub version")
+        try:
+            response = self._http_request(
+                url=self.url.version,
+                reauth=False,
+                timeout=self.http_timeout_default)
+            response_dict = response.json()
+        except (KeyError, ValueError, TypeError, IndexError):
+            raise exceptions.LhBaseException("LogicHub version could not be established")
         else:
-            url = self.url.custom_list_data
-            method = "GET"
-            payload = None
-        response = self._http_request(
-            method=method,
-            url=url,
-            params={"offset": offset, "limit": limit},
-            body=payload,
-            headers={'Content-Type': 'application/json'},
-        )
+            # Update version information any time this api call is run successfully
+            self.version_info = response_dict
+            self.version = float(re.match("m(.*)", self.version_info["version"]).group(1))
+        return response_dict
+
+    def get_workflow_by_id(self, workflow_id: int):
+        assert isinstance(workflow_id, int), "Workflow ID must be an integer"
+        response = self._http_request(method="GET", url=self.url.case_status_workflow_by_id.format(workflow_id))
+        return response.json()
+
+    def get_workflows(self):
+        response = self._http_request(method="GET", url=self.url.case_status_list_workflows)
+        return response.json()
+
+    # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
+    # ToDo Create an action method for this, and update the beta integration action to use it
+    def add_scoring_rule(self, rule_set, field_mappings, score: float or str):
+        rule_set = helpers.sanitize_input_rule_set_id(rule_set)
+        field_mappings = helpers.sanitize_input_rule_field_mappings(field_mappings)
+        score = helpers.sanitize_input_rule_score(score)
+        headers = {"Content-Type": "application/json"}
+        body = {
+            "method": "addRule",
+            "parameters": {
+                "values": [field_mappings],
+                "score": score
+            }
+        }
+        response = self._http_request(url=self.url.rule_set.format(rule_set), method="POST", body=body, headers=headers)
         results = response.json()
         try:
             results = results["result"]
         except Exception:
-            raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing custom lists")
+            raise exceptions.UnexpectedOutput("API response does not match the expected schema for adding a scoring rule")
         return results
 
     def execute_command(self, command_payload, limit=25):
@@ -914,15 +723,6 @@ class LogicHubAPI:
             _response = self._http_request(url=self.url.flow_export.format(_flow_id), test_response=False)
             self._save_export_to_disk(response=_response, export_folder=export_folder, resource_id=_flow_id, resource_name=_flow_name, file_info=_file_info)
 
-    def get_workflows(self):
-        response = self._http_request(method="GET", url=self.url.case_status_list_workflows)
-        return response.json()
-
-    def get_workflow_by_id(self, workflow_id: int):
-        assert isinstance(workflow_id, int), "Workflow ID must be an integer"
-        response = self._http_request(method="GET", url=self.url.case_status_workflow_by_id.format(workflow_id))
-        return response.json()
-
     def list_baselines(self):
         params = {"libraryView": "all"}
         body = {"filters": [], "offset": 0, "pageSize": 9999, "sortColumn": "lastUpdated", "sortOrder": "DESC"}
@@ -937,6 +737,10 @@ class LogicHubAPI:
         response = self._http_request(method="POST", url=self.url.connections, headers=headers, params=params, body=body)
         return response.json()
 
+    def list_dashboards_with_widgets(self):
+        response = self._http_request(url=self.url.dashboards_and_widgets)
+        return response.json()
+
     def list_fields(self, params: dict = None, **kwargs):
         params = params or {"systemFields": "true", "pageSize": 9999, "after": 0}
         response = self._http_request(method="GET", url=self.url.fields, params=params, **kwargs)
@@ -951,49 +755,47 @@ class LogicHubAPI:
         )
         return response.json()
 
-    def get_connection_status(self, connections: list):
-        """
-        Get status of one or more connections
-        :param connections: list of connection IDs, either as ints (like [6, 8]) or in full form (like ["connection-6", "connection-8"])
-        :return:
-        """
-        connection_list = []
-        if not isinstance(connections, (list, tuple)):
-            connections = [connections]
-        for connection in connections:
-            if isinstance(connection, str) and connection.startswith("connection-"):
-                connection_list.append(connection)
-            else:
-                connection_list.append(f"connection-{connection}")
+    def list_notebooks(self, limit=None, search_string=None):
+        limit = limit if limit and isinstance(limit, int) else 99999
+        body = {
+            "filters": [],
+            "offset": 0,
+            "pageSize": limit,
+            "sortColumn": "name",
+            "sortOrder": "ASC",
+        }
+        if search_string:
+            body['filters'] = [{"searchText": search_string}]
+        response = self._http_request(
+            url=self.url.notebooks_list,
+            method="POST",
+            headers={"Content-Type": "application/json"},
+            params={"libraryView": "all"},
+            body=body
+        )
+        response = response.json()
+        # Sort the results by notebook ID
+        self.notebooks = response["result"]["data"]["data"] = helpers.sort_notebook_objects_by_id(response["result"]["data"]["data"])
+        return response
 
+    def list_streams(self, search_text: str = None, filters: list = None, limit: int = 25, offset: int = 0):
+        params = {"libraryView": "all"}
         headers = {"Content-Type": "application/json"}
-        # response = self._http_request(method="POST", url=self.url.connection_status, headers=headers, body=json.dumps(connection_list))
-        response = self._http_request(method="POST", url=self.url.connection_status, headers=headers, body=connection_list)
-        return response.json()
 
-    # ToDo Revisit and finish this
-    # def get_alerts(self, advanced_filter=None, limit=100):
-    #     limit = int(limit) if limit is not None else 100_000
-    #     body = {
-    #         "pageNumber": 0,
-    #         "pageSize": limit,
-    #         "query": advanced_filter,
-    #         "sortCol": "id",
-    #         "sortOrder": "desc"
-    #     }
-    #     connection_list = []
-    #     if not isinstance(connections, (list, tuple)):
-    #         connections = [connections]
-    #     for connection in connections:
-    #         if isinstance(connection, str) and connection.startswith("connection-"):
-    #             connection_list.append(connection)
-    #         else:
-    #             connection_list.append(f"connection-{connection}")
-    #
-    #     headers = {"Content-Type": "application/json"}
-    #     # response = self._http_request(method="POST", url=self.url.connection_status, headers=headers, body=json.dumps(connection_list))
-    #     response = self._http_request(method="POST", url=self.url.connection_status, headers=headers, body=connection_list)
-    #     return response.json()
+        filters = filters or []
+        if search_text:
+            filters.append({"searchText": search_text})
+
+        body_dict = {
+            "filters": filters,
+            "offset": offset,
+            "pageSize": limit,
+            "sortColumn": "name",
+            "sortOrder": "ASC"
+        }
+
+        response = self._http_request(method="POST", url=self.url.streams, body=body_dict, headers=headers, params=params)
+        return response.json()
 
     def reprocess_batch(self, batch_id):
         """
@@ -1023,12 +825,12 @@ class LogicHubAPI:
     #         alert_ids = [alert_ids]
     #     updated_alerts = []
     #     for i in alert_ids:
-    #         i = self._format_alert_id(i)
+    #         i = helpers.format_alert_id(i)
     #         if i:
     #             updated_alerts.append(i)
     #     alert_ids = updated_alerts
     #
-    #     case_id = self._format_case_id(case_id)
+    #     case_id = helpers.format_case_id_with_prefix(case_id, self.case_prefix)
     #
     #     field_id = self.system_field_lh_linked_alerts.get('id')
     #     body = {"fields": [{"id": field_id, "value": alert_ids}]}
@@ -1048,12 +850,8 @@ class LogicHubAPI:
         self.case_prefix = output['result']
         return output
 
-    @staticmethod
-    def __sort_notebook_objects_by_id(notebooks):
-        return sorted(notebooks, key=lambda x: (x['id']['id']))
-
     def case_list_attached_notebooks(self, case_id, results_only=False):
-        case_id = self._format_case_id(case_id)
+        case_id = helpers.format_case_id_with_prefix(case_id, self.case_prefix)
         body = {
             "id": case_id,
             "key": "case",
@@ -1066,83 +864,16 @@ class LogicHubAPI:
         )
         response = response.json()
         # Sort the results by notebook ID
-        results = response['result'] = self.__sort_notebook_objects_by_id(response['result'])
+        results = response['result'] = helpers.sort_notebook_objects_by_id(response['result'])
         if results_only:
             for n in range(len(results)):
                 results[n]['id']['id'] = int(results[n]['id']['id'])
             return results
         return response
 
-    def notebooks_list(self, limit=None, search_string=None):
-        limit = limit if limit and isinstance(limit, int) else 99999
-        body = {
-            "filters": [],
-            "offset": 0,
-            "pageSize": limit,
-            "sortColumn": "name",
-            "sortOrder": "ASC",
-        }
-        if search_string:
-            body['filters'] = [{"searchText": search_string}]
-        response = self._http_request(
-            url=self.url.notebooks_list,
-            method="POST",
-            headers={"Content-Type": "application/json"},
-            params={"libraryView": "all"},
-            body=body
-        )
-        response = response.json()
-        # Sort the results by notebook ID
-        self.notebooks = response["result"]["data"]["data"] = self.__sort_notebook_objects_by_id(response["result"]["data"]["data"])
-        return response
-
-    @staticmethod
-    def _format_alert_id(alert_id):
-        if isinstance(alert_id, str):
-            if not re.match(r'^(?:alert-)?\d+$', alert_id):
-                raise exceptions.Formatting.InvalidAlertIdFormat(alert_id)
-            alert_id = re.sub(r'\D+', '', alert_id)
-        return int(alert_id)
-
-    def _format_case_id(self, case_id):
-        if not case_id:
-            raise ValueError("Case ID cannot be blank")
-        case_id = str(case_id).strip()
-        if '-' not in case_id:
-            case_id = f"{self.case_prefix}-{case_id}"
-        return case_id
-
-    @staticmethod
-    def _format_notebook_ids(notebook_ids):
-        if not isinstance(notebook_ids, list):
-            notebook_ids = [notebook_ids]
-        final_notebooks = []
-        for input_value in notebook_ids:
-            if isinstance(input_value, dict):
-                # In case a raw notebook object is passed, drill into the 'id' field for the part we need
-                if isinstance(input_value, dict) and isinstance(input_value.get('id'), dict):
-                    input_value = input_value['id']
-                if not input_value or 'id' not in input_value.keys() or not isinstance(input_value['id'], (int, str)):
-                    raise exceptions.Formatting.InvalidNotebookIdFormat(input_value)
-                final_notebooks.append({'key': 'notebook', 'id': int(input_value['id'])})
-            else:
-                try:
-                    final_notebooks.append({'key': 'notebook', 'id': int(input_value)})
-                except (ValueError, TypeError):
-                    raise exceptions.Formatting.InvalidNotebookIdFormat(input_value)
-        return final_notebooks
-
-    @staticmethod
-    def _format_stream_id(alert_id):
-        if isinstance(alert_id, str):
-            if not re.match(r'^(?:stream-)?\d+$', alert_id):
-                raise exceptions.Formatting.InvalidStreamIdFormat(alert_id)
-            alert_id = re.sub(r'\D+', '', alert_id)
-        return int(alert_id)
-
     def case_overwrite_attached_notebooks(self, case_id, notebooks):
-        notebooks = self._format_notebook_ids(notebooks)
-        case_id = self._format_case_id(case_id)
+        notebooks = helpers.format_notebook_ids(notebooks)
+        case_id = helpers.format_case_id_with_prefix(case_id, self.case_prefix)
         body = {
             "notebookAttachmentIds": notebooks,
             "notebookAttachedEntityId": {
@@ -1190,7 +921,7 @@ class LogicHubAPI:
         return response.json()
 
     def alert_fetch(self, alert_id, return_raw=False, **kwargs):
-        alert_id = self._format_alert_id(alert_id)
+        alert_id = helpers.format_alert_id(alert_id)
         url = self.url.alert_fetch.format(alert_id)
         # Send request, but disable immediate testing in order to do specialized testing first
         response = self._http_request(method="GET", url=url, test_response=False, **kwargs)
@@ -1242,12 +973,12 @@ class LogicHubAPI:
 
     def stream_pause(self, stream_id, **kwargs):
         # Sanitize stream ID
-        body = [f"stream-{self._format_stream_id(stream_id)}"]
+        body = [f"stream-{helpers.format_stream_id(stream_id)}"]
         response = self._http_request(method="POST", url=self.url.stream_pause, body=body, **kwargs)
         return response.json()
 
     def stream_resume(self, stream_id, **kwargs):
         # Sanitize stream ID
-        body = [f"stream-{self._format_stream_id(stream_id)}"]
+        body = [f"stream-{helpers.format_stream_id(stream_id)}"]
         response = self._http_request(method="POST", url=self.url.stream_resume, body=body, **kwargs)
         return response.json()
